@@ -3,14 +3,12 @@ __all__ = (
     "Authenticator",
     "AuthMessage",
     "Config",
-    "generate_certs",
     "smtpd",
     "SMTPDFix",
 )
-__version__ = "0.3.2"
+__version__ = "0.4.0"
 
 from .authenticator import Authenticator
-from .certs import generate_certs
 from .configuration import Config
 from .controller import AuthController
 from .fixture import SMTPDFix, smtpd
